@@ -3,38 +3,50 @@ import ast
 import dis
 import sys
 import warnings
-from typing import Union, Tuple, Any, Optional
+from typing import Callable, Union, Tuple, Any, Optional
 from types import FrameType, CodeType
 from collections import namedtuple as standard_namedtuple
+from functools import wraps
 from functools import lru_cache
 
 import executing
 
-__version__ = "0.5.3"
+__version__ = "0.5.5"
 __all__ = [
-    "VarnameRetrievingError", "varname", "will",
+    "VarnameRetrievingError", "varname", "will", "inject_varname",
     "inject", "nameof", "namedtuple", "Wrapper", "debug"
 ]
 
 class VarnameRetrievingError(Exception):
     """When failed to retrieve the varname"""
 
-def varname(caller: int = 1, raise_exc: bool = True) -> Optional[str]:
+def varname(
+        caller: int = 1,
+        multi_vars: bool = False,
+        raise_exc: bool = True
+) -> Optional[Union[str, Tuple[Union[str, tuple]]]]:
     """Get the variable name that assigned by function/class calls
 
     Args:
         caller: The call stack index, indicating where this function
             is called relative to where the variable is finally retrieved
+        multi_vars: Whether allow multiple variables on left-hand side (LHS).
+            If `True`, this function returns a tuple of the variable names,
+            even there is only one variable on LHS.
+            If `False`, and multiple variables on LHS, a
+            `VarnameRetrievingError` will be raised.
         raise_exc: Whether we should raise an exception if failed
             to retrieve the name.
 
     Returns:
         The variable name, or `None` when `raise_exc` is `False` and
             we failed to retrieve the variable name.
+        A tuple or a hierarchy (tuple of tuples) of variable names
+            when `multi_vars` is `True`.
 
     Raises:
-        VarnameRetrievingError: When there is invalid variable used on the
-            left of the assign node. (e.g: `a.b = func()`) or
+        VarnameRetrievingError: When there is invalid variables or
+            invalid number of variables used on the LHS; or
             when we are unable to retrieve the variable name and `raise_exc`
             is set to `True`.
 
@@ -63,7 +75,21 @@ def varname(caller: int = 1, raise_exc: bool = True) -> Optional[str]:
                       "on the very left will be used.",
                       UserWarning)
     target = node.targets[0]
-    return _node_name(target)
+
+    names = _node_name(target)
+
+    if not isinstance(names, tuple):
+        names = (names, )
+
+    if multi_vars:
+        return names
+
+    if len(names) > 1:
+        raise VarnameRetrievingError(
+            f"Expecting a single variable on left-hand side, got {len(names)}."
+        )
+
+    return names[0]
 
 def will(caller: int = 1, raise_exc: bool = True) -> Optional[str]:
     """Detect the attribute name right immediately after a function call.
@@ -126,6 +152,62 @@ def will(caller: int = 1, raise_exc: bool = True) -> Optional[str]:
     # ast.Attribute
     return node.attr
 
+def inject_varname(
+        cls: type = None, *,
+        caller: int = 1,
+        multi_vars: bool = False,
+        raise_exc: bool = True
+) -> Union[type, Callable[[type], type]]:
+    """A decorator to inject __varname__ attribute to a class
+
+    Args:
+        caller: The call stack index, indicating where this class
+            is instantiated relative to where the variable is finally retrieved
+        multi_vars: Whether allow multiple variables on left-hand side (LHS).
+            If `True`, this function returns a tuple of the variable names,
+            even there is only one variable on LHS.
+            If `False`, and multiple variables on LHS, a
+            `VarnameRetrievingError` will be raised.
+        raise_exc: Whether we should raise an exception if failed
+            to retrieve the name.
+
+    Examples:
+        >>> @inject_varname
+        >>> class Foo: pass
+        >>> foo = Foo()
+        >>> # foo.__varname__ == 'foo'
+
+    Returns:
+        The wrapper function or the class itself if it is specified explictly.
+    """
+    if cls is not None:
+        # Used as @inject_varname directly
+        return inject_varname(
+            caller=caller,
+            multi_vars=multi_vars,
+            raise_exc=raise_exc
+        )(cls)
+
+    # Used as @inject_varname(multi_vars=..., raise_exc=...)
+    def wrapper(cls):
+        """The wrapper function to wrap a class and inject `__varname__`"""
+        orig_init = cls.__init__
+
+        @wraps(cls.__init__)
+        def wrapped_init(self, *args, **kwargs):
+            """Wrapped init function to replace the original one"""
+            self.__varname__ = varname(
+                caller=caller,
+                multi_vars=multi_vars,
+                raise_exc=raise_exc
+            )
+            orig_init(self, *args, **kwargs)
+
+        cls.__init__ = wrapped_init
+        return cls
+
+    return wrapper
+
 def inject(obj: object) -> object:
     """Inject attribute `__varname__` to an object
 
@@ -156,6 +238,9 @@ def inject(obj: object) -> object:
     Returns:
         The object with __varname__ injected
     """
+    warnings.warn("Function inject will be removed in 0.6.0. Use "
+                  "varname.inject_varname to decorate your class.",
+                  DeprecationWarning)
     vname = varname()
     try:
         setattr(obj, '__varname__', vname)
@@ -408,7 +493,7 @@ def _lookfor_parent_assign(node: ast.AST) -> Optional[ast.Assign]:
             return node
     return None
 
-def _node_name(node: ast.AST) -> str:
+def _node_name(node: ast.AST) -> Optional[Union[str, Tuple[Union[str, tuple]]]]:
     """Get the node node name.
 
     Raises VarnameRetrievingError when failed
@@ -417,6 +502,8 @@ def _node_name(node: ast.AST) -> str:
         return node.id
     if isinstance(node, ast.Attribute):
         return node.attr
+    if isinstance(node, (ast.List, ast.Tuple)):
+        return tuple(_node_name(elem) for elem in node.elts)
 
     raise VarnameRetrievingError(
         f"Can only get name of a variable or attribute, "
